@@ -17,7 +17,7 @@
 #include <vector>
 #include <wx/textctrl.h> // to inherit
 #include "commands/CommandManagerWindowClasses.h"
-#include "widgets/wxPanelWrapper.h" // to inherit
+#include "wxPanelWrapper.h" // to inherit
 #include "Observer.h"
 
 class AudacityProject;
