@@ -8,22 +8,28 @@
 
 **********************************************************************/
 
-#include "Experimental.h"
+
+
 
 #ifndef __AUDACITY_MIXER_BOARD__
 #define __AUDACITY_MIXER_BOARD__
 
-#include <wx/frame.h>
-#include <wx/bmpbuttn.h>
-#include <wx/hashmap.h>
-#include <wx/image.h>
-#include <wx/scrolwin.h>
-#include <wx/statbmp.h>
-#include <wx/stattext.h>
+#include <wx/frame.h> // to inherit
+#include <wx/scrolwin.h> // to inherit
 
-#include "widgets/AButton.h"
-#include "widgets/ASlider.h"
-#include "widgets/wxPanelWrapper.h"
+#include "widgets/ASlider.h" // to inherit
+#include "commands/CommandManagerWindowClasses.h"
+
+#include "Observer.h"
+#include "Prefs.h"
+
+class wxArrayString;
+class wxBitmapButton;
+class wxImage;
+class wxMemoryDC;
+class AButton;
+struct AudioIOEvent;
+struct TrackListEvent;
 
 // containment hierarchy:
 //    MixerBoardFrame -> MixerBoard -> MixerBoardScrolledWindow -> MixerTrackCluster(s)
@@ -36,14 +42,10 @@ class MixerTrackSlider final : public ASlider
 public:
    MixerTrackSlider(wxWindow * parent,
                      wxWindowID id,
-                     const wxString &name,
+                     const TranslatableString &name,
                      const wxPoint & pos,
                      const wxSize & size,
-                     int style = FRAC_SLIDER,
-                     bool popup = true,
-                     bool canUseShift = true,
-                     float stepValue = STEP_CONTINUOUS,
-                     int orientation = wxHORIZONTAL);
+                     const ASlider::Options &options = ASlider::Options{});
    virtual ~MixerTrackSlider() {}
 
    void OnMouseEvent(wxMouseEvent & event);
@@ -60,7 +62,7 @@ public:
 
 
 class AudacityProject;
-class Meter;
+class MeterPanel;
 class MixerBoard;
 
 class Track;
@@ -88,7 +90,7 @@ public:
    NoteTrack *GetNote() const;
 #endif
 
-   void UpdatePrefs();
+   //void UpdatePrefs();
 
    void HandleResize(); // For wxSizeEvents, update gain slider and meter.
 
@@ -100,16 +102,7 @@ public:
 
    void ResetMeter(const bool bResetClipping);
 
-   // These are used by TrackPanel for synchronizing control states.
-   void UpdateForStateChange(); // Update the controls that can be affected by state change.
-   void UpdateName();
-   void UpdateMute();
-   void UpdateSolo();
-   void UpdatePan();
-   void UpdateGain();
-#ifdef EXPERIMENTAL_MIDI_OUT
-   void UpdateVelocity();
-#endif
+   void UpdateForStateChange();
    void UpdateMeter(const double t0, const double t1);
 
 private:
@@ -150,14 +143,12 @@ private:
 #ifdef EXPERIMENTAL_MIDI_OUT
    MixerTrackSlider* mSlider_Velocity;
 #endif
-   Meter* mMeter;
+   wxWeakRef<MeterPanel> mMeter;
 
 public:
    DECLARE_EVENT_TABLE()
 };
 
-WX_DEFINE_ARRAY(MixerTrackCluster*, MixerTrackClusterArray);
-
 
 class MusicalInstrument
 {
@@ -169,7 +160,7 @@ public:
    wxArrayString  mKeywords;
 };
 
-using MusicalInstrumentArray = std::vector<movable_ptr<MusicalInstrument>>;
+using MusicalInstrumentArray = std::vector<std::unique_ptr<MusicalInstrument>>;
 
 
 
@@ -201,7 +192,7 @@ public:
 class MixerBoardFrame;
 class TrackList;
 
-class MixerBoard final : public wxWindow
+class MixerBoard final : public wxWindow, private PrefsListener
 {
    friend class MixerBoardFrame;
 
@@ -210,55 +201,44 @@ public:
                wxFrame* parent,
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxDefaultSize);
-   virtual ~MixerBoard();
 
-   void UpdatePrefs();
+   void UpdatePrefs() override;
 
    // Add clusters for any tracks we're not yet showing.
-   // Update pointers for tracks we're aleady showing.
+   // Update pointers for tracks we're already showing.
    void UpdateTrackClusters();
 
    int GetTrackClustersWidth();
-   void MoveTrackCluster(const PlayableTrack* pTrack, bool bUp); // Up in TrackPanel is left in MixerBoard.
-   void RemoveTrackCluster(const PlayableTrack* pTrack);
-   void RemoveTrackCluster(size_t nIndex);
 
 
    wxBitmap* GetMusicalInstrumentBitmap(const Track *pTrack);
 
    bool HasSolo();
 
-   void RefreshTrackCluster(const PlayableTrack* pTrack, bool bEraseBackground = true);
    void RefreshTrackClusters(bool bEraseBackground = true);
    void ResizeTrackClusters();
 
-   void ResetMeters(const bool bResetClipping);
-
-   void UpdateName(const PlayableTrack* pTrack);
-   void UpdateMute(const PlayableTrack* pTrack = NULL); // NULL means update for all tracks.
-   void UpdateSolo(const PlayableTrack* pTrack = NULL); // NULL means update for all tracks.
-   void UpdatePan(const PlayableTrack* pTrack = NULL); // NULL means update for all tracks.
-   void UpdateGain(const PlayableTrack* pTrack);
-#ifdef EXPERIMENTAL_MIDI_OUT
-   void UpdateVelocity(const PlayableTrack* pTrack);
-#endif
-
    void UpdateMeters(const double t1, const bool bLoopedPlay);
 
    void UpdateWidth();
 
 private:
-   void MakeButtonBitmap( wxMemoryDC & dc, wxBitmap & bitmap, 
-      wxRect & bev, const wxString & str, bool up );
+   void ResetMeters(const bool bResetClipping);   
+   void RemoveTrackCluster(size_t nIndex);
+   void MakeButtonBitmap( wxMemoryDC & dc, wxBitmap & bitmap,
+      wxRect & bev, const TranslatableString & str, bool up );
    void CreateMuteSoloImages();
    int FindMixerTrackCluster(const PlayableTrack* pTrack,
                               MixerTrackCluster** hMixerTrackCluster) const;
    void LoadMusicalInstruments();
 
    // event handlers
+   void OnPaint(wxPaintEvent& evt);
    void OnSize(wxSizeEvent &evt);
-   void OnTimer(wxCommandEvent &event);
-
+   void OnTimer(Observer::Message);
+   void OnTrackSetChanged();
+   void OnTrackChanged(const TrackListEvent &event);
+   void OnStartStop(AudioIOEvent);
 
 public:
    // mute & solo button images: Create once and store on MixerBoard for use in all MixerTrackClusters.
@@ -269,26 +249,35 @@ public:
    int mMuteSoloWidth;
 
 private:
+   Observer::Subscription mPlaybackScrollerSubscription,
+      mTrackPanelSubscription,
+      mAudioIOSubscription;
+
    // Track clusters are maintained in the same order as the WaveTracks.
-   MixerTrackClusterArray     mMixerTrackClusters;
+   std::vector<MixerTrackCluster*> mMixerTrackClusters;
 
    MusicalInstrumentArray     mMusicalInstruments;
    AudacityProject*           mProject;
    MixerBoardScrolledWindow*  mScrolledWindow; // Holds the MixerTrackClusters and handles scrolling.
    double                     mPrevT1;
    TrackList*                 mTracks;
+   bool                       mUpToDate{ false };
 
 public:
    DECLARE_EVENT_TABLE()
 };
 
 
-class MixerBoardFrame final : public wxFrame
+class MixerBoardFrame final
+   : public wxFrame
+   , public TopLevelKeystrokeHandlingWindow
 {
 public:
    MixerBoardFrame(AudacityProject* parent);
    virtual ~MixerBoardFrame();
 
+   void Recreate(AudacityProject *pProject);
+
 private:
    // event handlers
    void OnCloseWindow(wxCloseEvent &WXUNUSED(event));
@@ -296,6 +285,10 @@ private:
    void OnSize(wxSizeEvent &evt);
    void OnKeyEvent(wxKeyEvent &evt);
 
+   void SetWindowTitle();
+
+   Observer::Subscription mTitleChangeSubscription;
+   AudacityProject *mProject;
 public:
    MixerBoard* mMixerBoard;
 
