@@ -18,8 +18,10 @@ Paul Licameli split from AudacityProject.cpp
 #include "Menus.h"
 #include "Project.h"
 #include "ProjectAudioIO.h"
+#include "ProjectFileIO.h"
 #include "ProjectWindows.h"
 #include "ProjectStatus.h"
+#include "ProjectSnap.h"
 #include "RefreshCode.h"
 #include "TrackPanelMouseEvent.h"
 #include "TrackPanelAx.h"
@@ -33,11 +35,10 @@ Paul Licameli split from AudacityProject.cpp
 #include "toolbars/ToolManager.h"
 #include "tracks/ui/Scrubbing.h"
 #include "tracks/ui/TrackView.h"
-#include "widgets/wxPanelWrapper.h"
-#include "widgets/WindowAccessible.h"
+#include "wxPanelWrapper.h"
+#include "WindowAccessible.h"
 
 #include "ThemedWrappers.h"
-#include "RealtimeEffectPanel.h"
 
 #include <wx/app.h>
 #include <wx/display.h>
@@ -482,7 +483,7 @@ mutable -> unsigned {
 #endif
 
       wxCoord xTrackEnd = viewInfo.TimeToPosition( audioEndTime );
-      viewInfo.ZoomBy(pow(2.0, steps));
+      viewInfo.ZoomBy(pow(2.0, steps/4.0));
 
       double new_center_h = viewInfo.PositionToTime(xx, trackLeftEdge);
       viewInfo.h += (center_h - new_center_h);
@@ -605,7 +606,6 @@ ProjectWindow::ProjectWindow(wxWindow * parent, wxWindowID id,
    mNextWindowID = NextID;
    
    constexpr auto EffectsPanelMaxWidth { 500 };
-   constexpr auto EffectsPanelMinWidth { 255 };
    constexpr auto TrackPanelMinWidth { 250 };
 
    // Two sub-windows need to be made before Init(),
@@ -655,18 +655,6 @@ ProjectWindow::ProjectWindow(wxWindow * parent, wxWindowID id,
 
    mContainerWindow->Initialize(mTrackListWindow);
 
-   auto effectsPanel = safenew ThemedWindowWrapper<RealtimeEffectPanel>(
-      project, mContainerWindow, wxID_ANY);
-   effectsPanel->SetMinSize({EffectsPanelMinWidth, -1});
-   effectsPanel->SetName(_("Realtime effects"));
-   effectsPanel->SetBackgroundColorIndex(clrMedium);
-   effectsPanel->Hide();//initially hidden
-   effectsPanel->Bind(wxEVT_CLOSE_WINDOW, [this](wxCloseEvent&)
-   {
-      HideEffectsPanel();
-   });
-   mEffectsWindow = effectsPanel;
-
 #ifdef EXPERIMENTAL_DA2
    mTrackListWindow->SetBackgroundColour(theTheme.Colour( clrMedium ));
 #endif
@@ -711,17 +699,15 @@ ProjectWindow::ProjectWindow(wxWindow * parent, wxWindowID id,
    mThemeChangeSubscription =
       theTheme.Subscribe(*this, &ProjectWindow::OnThemeChange);
 
-   mFocusChangeSubscription = TrackFocus::Get(project)
-      .Subscribe([this](const TrackFocusChangeMessage& msg) {
-         if(mEffectsWindow->IsShown())
-         {
-            if (auto project = FindProject()) {
-               auto& trackFocus = TrackFocus::Get(*project);
-               ShowEffectsPanel(trackFocus.Get(), false);
-            }
-         }
-      });
+   // Subscribe to title changes published by ProjectFileIO
+   mTitleChangeSubcription = ProjectFileIO::Get(project)
+      .Subscribe(*this, &ProjectWindow::OnProjectTitleChange);
 
+   // Subscribe to snapping changes
+   mSnappingChangedSubscription =
+      ProjectSnap::Get(project).Subscribe([this](auto) { RedrawProject(); });
+   // And also establish my initial consistency with it
+   this->OnProjectTitleChange(ProjectFileIOMessage::ProjectTitleChange);
 }
 
 ProjectWindow::~ProjectWindow()
@@ -796,12 +782,10 @@ void ProjectWindow::OnThemeChange(ThemeChangeMessage message)
       return;
    this->ApplyUpdatedTheme();
    auto &toolManager = ToolManager::Get( project );
-   for( int ii = 0; ii < ToolBarCount; ++ii )
-   {
-      ToolBar *pToolBar = toolManager.GetToolBar(ii);
+   toolManager.ForEach([](auto pToolBar){
       if( pToolBar )
          pToolBar->ReCreateButtons();
-   }
+   });
 }
 
 void ProjectWindow::UpdatePrefs()
@@ -1324,17 +1308,12 @@ bool ProjectWindow::IsIconized() const
    return mIconized;
 }
 
-wxWindow* ProjectWindow::GetEffectsWindow() noexcept
-{
-   return mEffectsWindow;
-}
-
 wxWindow* ProjectWindow::GetTrackListWindow() noexcept
 {
    return mTrackListWindow;
 }
 
-wxWindow* ProjectWindow::GetContainerWindow() noexcept
+wxSplitterWindow* ProjectWindow::GetContainerWindow() noexcept
 {
    return mContainerWindow;
 }
@@ -1543,6 +1522,21 @@ void ProjectWindow::OnUndoReset()
    // RedrawProject();  // Should we do this here too?
 }
 
+void ProjectWindow::OnProjectTitleChange(ProjectFileIOMessage message)
+{
+   if (message == ProjectFileIOMessage::ProjectTitleChange) {
+      auto pProject = FindProject();
+      if (!pProject)
+         return;
+      auto &project = *pProject;
+      const auto &name = ProjectFileIO::Get(project).GetProjectTitle();
+      if (name != GetTitle()) {
+         SetTitle(name);
+         SetName(name); // to make the nvda screen reader read the correct title
+      }
+   }
+}
+
 void ProjectWindow::OnScroll(wxScrollEvent & WXUNUSED(event))
 {
    auto pProject = FindProject();
@@ -2019,50 +2013,6 @@ void ProjectWindow::DoZoomFit()
    window.TP_ScrollWindow(start);
 }
 
-void ProjectWindow::ShowEffectsPanel(Track* track, bool focus)
-{
-   if(track == nullptr)
-   {
-      mEffectsWindow->ResetTrack();
-      return;
-   }
-
-   wxWindowUpdateLocker freeze(this);
-
-   mEffectsWindow->SetTrack(track->shared_from_this());
-
-   if(mContainerWindow->GetWindow1() != mEffectsWindow)
-   {
-      //Restore previous effects window size
-      mContainerWindow->SplitVertically(
-         mEffectsWindow,
-         mTrackListWindow,
-         mEffectsWindow->GetSize().GetWidth());
-   }
-   if(focus)
-      mEffectsWindow->SetFocus();
-   Layout();
-}
-
-void ProjectWindow::HideEffectsPanel()
-{
-   wxWindowUpdateLocker freeze(this);
-
-   if(mContainerWindow->GetWindow2() == nullptr)
-      //only effects panel is present, restore split positions before removing effects panel
-      //Workaround: ::Replace and ::Initialize do not work here...
-      mContainerWindow->SplitVertically(mEffectsWindow, mTrackListWindow);
-
-   mContainerWindow->Unsplit(mEffectsWindow);
-   mTrackListWindow->SetFocus();
-   Layout();
-}
-
-bool ProjectWindow::IsEffectsPanelShown()
-{
-   return mEffectsWindow->IsShown();
-}
-
 static ToolManager::TopPanelHook::Scope scope {
 []( wxWindow &window ){
    auto pProjectWindow = dynamic_cast< ProjectWindow* >( &window );
