@@ -17,112 +17,89 @@
 \brief FindClippingDialog used with EffectFindClipping
 
 *//*******************************************************************/
-
-
-#include "../Audacity.h"
 #include "FindClipping.h"
+#include "AnalysisTracks.h"
+#include "EffectEditor.h"
+#include "LoadEffects.h"
 
 #include <math.h>
 
-#include <wx/intl.h>
 
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "../widgets/valnum.h"
-#include "../widgets/ErrorDialog.h"
+#include "AudacityMessageBox.h"
 
 #include "../LabelTrack.h"
-#include "../WaveTrack.h"
-#include "../MemoryX.h"
+#include "WaveTrack.h"
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name    Type  Key               Def   Min   Max      Scale
-Param( Start,  int,  wxT("Duty Cycle"), 3,    1,    INT_MAX, 1   );
-Param( Stop,   int,  wxT("Duty Cycle"), 3,    1,    INT_MAX, 1   );
+const EffectParameterMethods& EffectFindClipping::Parameters() const
+{
+   static CapturedParameters<EffectFindClipping,
+      Start, Stop
+   > parameters;
+   return parameters;
+}
+
+const ComponentInterfaceSymbol EffectFindClipping::Symbol
+{ XO("Find Clipping") };
+
+namespace{ BuiltinEffectsModule::Registration< EffectFindClipping > reg; }
 
 EffectFindClipping::EffectFindClipping()
 {
-   mStart = DEF_Start;
-   mStop = DEF_Stop;
+   Parameters().Reset(*this);
 }
 
 EffectFindClipping::~EffectFindClipping()
 {
 }
 
-// IdentInterface implementation
+// ComponentInterface implementation
 
-wxString EffectFindClipping::GetSymbol()
+ComponentInterfaceSymbol EffectFindClipping::GetSymbol() const
 {
-   return FINDCLIPPING_PLUGIN_SYMBOL;
+   return Symbol;
 }
 
-wxString EffectFindClipping::GetDescription()
+TranslatableString EffectFindClipping::GetDescription() const
 {
-   return _("Creates labels where clipping is detected");
+   return XO("Creates labels where clipping is detected");
 }
 
-wxString EffectFindClipping::ManualPage()
+ManualPageID EffectFindClipping::ManualPage() const
 {
-   return wxT("Find_Clipping");
+   return L"Find_Clipping";
 }
 
-// EffectIdentInterface implementation
+// EffectDefinitionInterface implementation
 
-EffectType EffectFindClipping::GetType()
+EffectType EffectFindClipping::GetType() const
 {
    return EffectTypeAnalyze;
 }
 
-// EffectClientInterface implementation
-
-bool EffectFindClipping::GetAutomationParameters(EffectAutomationParameters & parms)
-{
-   parms.Write(KEY_Start, mStart);
-   parms.Write(KEY_Stop, mStop);
-
-   return true;
-}
-
-bool EffectFindClipping::SetAutomationParameters(EffectAutomationParameters & parms)
-{
-   ReadAndVerifyInt(Start);
-   ReadAndVerifyInt(Stop);
-
-   mStart = Start;
-   mStop = Stop;
-
-   return true;
-}
-
 // Effect implementation
 
-bool EffectFindClipping::Process()
+bool EffectFindClipping::Process(EffectInstance &, EffectSettings &)
 {
    std::shared_ptr<AddedAnalysisTrack> addedTrack;
-   Maybe<ModifiedAnalysisTrack> modifiedTrack;
+   std::optional<ModifiedAnalysisTrack> modifiedTrack;
    const wxString name{ _("Clipping") };
 
-   LabelTrack *lt = NULL;
-   TrackListOfKindIterator iter(Track::Label, inputTracks());
-   for (Track *t = iter.First(); t; t = iter.Next()) {
-      if (t->GetName() == name) {
-         lt = (LabelTrack *)t;
-         break;
-      }
-   }
+   auto clt = *inputTracks()->Any< const LabelTrack >().find_if(
+      [&]( const Track *track ){ return track->GetName() == name; } );
 
-   if (!lt)
-      addedTrack = (AddAnalysisTrack(name)), lt = addedTrack->get();
+   LabelTrack *lt{};
+   if (!clt)
+      addedTrack = (AddAnalysisTrack(*this, name)), lt = addedTrack->get();
    else
-      modifiedTrack.create(ModifyAnalysisTrack(lt, name)), lt = modifiedTrack->get();
+      modifiedTrack.emplace(ModifyAnalysisTrack(*this, clt, name)),
+      lt = modifiedTrack->get();
 
    int count = 0;
 
    // JC: Only process selected tracks.
-   SelectedTrackListOfKindIterator waves(Track::Wave, inputTracks());
-   WaveTrack *t = (WaveTrack *) waves.First();
-   while (t) {
+   for (auto t : inputTracks()->Selected< const WaveTrack >()) {
       double trackStart = t->GetStartTime();
       double trackEnd = t->GetEndTime();
       double t0 = mT0 < trackStart ? trackStart : mT0;
@@ -139,7 +116,6 @@ bool EffectFindClipping::Process()
       }
 
       count++;
-      t = (WaveTrack *) waves.Next();
    }
 
    // No cancellation, so commit the addition of the track.
@@ -173,7 +149,8 @@ bool EffectFindClipping::ProcessOne(LabelTrack * lt,
       buffer.reinit(blockSize);
    }
    catch( const std::bad_alloc & ) {
-      Effect::MessageBox(_("Requested value exceeds memory capacity."));
+      EffectUIServices::DoMessageBox(*this,
+         XO("Requested value exceeds memory capacity."));
       return false;
    }
 
@@ -194,7 +171,7 @@ bool EffectFindClipping::ProcessOne(LabelTrack * lt,
 
          block = limitSampleBufferSize( blockSize, len - s );
 
-         wt->Get((samplePtr)buffer.get(), floatSample, start + s, block);
+         wt->GetFloats(buffer.get(), start + s, block);
          ptr = buffer.get();
       }
 
@@ -218,8 +195,7 @@ bool EffectFindClipping::ProcessOne(LabelTrack * lt,
             if (stoprun >= mStop) {
                lt->AddLabel(SelectedRegion(startTime,
                                           wt->LongSamplesToTime(start + s - mStop)),
-                           wxString::Format(wxT("%lld of %lld"), startrun.as_long_long(), (samps - mStop).as_long_long()),
-                           -2);
+                           wxString::Format(wxT("%lld of %lld"), startrun.as_long_long(), (samps - mStop).as_long_long()));
                startrun = 0;
                stoprun = 0;
                samps = 0;
@@ -237,34 +213,44 @@ bool EffectFindClipping::ProcessOne(LabelTrack * lt,
    return bGoodResult;
 }
 
-void EffectFindClipping::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectEditor> EffectFindClipping::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access,
+   const EffectOutputs *)
 {
+   mUIParent = S.GetParent();
+   DoPopulateOrExchange(S, access);
+   return nullptr;
+}
+
+void EffectFindClipping::DoPopulateOrExchange(
+   ShuttleGui & S, EffectSettingsAccess &access)
+{
+   mpAccess = access.shared_from_this();
    S.StartMultiColumn(2, wxALIGN_CENTER);
    {
-      IntegerValidator<int> vldStart(&mStart);
-      vldStart.SetMin(MIN_Start);
-      S.TieTextBox(_("Start threshold (samples):"),
-                   mStart,
-                   10)->SetValidator(vldStart);
+      S
+         .Validator<IntegerValidator<int>>(
+            &mStart, NumValidatorStyle::DEFAULT, Start.min)
+         .TieTextBox(XXO("&Start threshold (samples):"), mStart, 10);
 
-      IntegerValidator<int> vldStop(&mStop);
-      vldStop.SetMin(MIN_Stop);
-      S.TieTextBox(_("Stop threshold (samples):"),
-                   mStop,
-                   10)->SetValidator(vldStop);
+      S
+         .Validator<IntegerValidator<int>>(
+            &mStop, NumValidatorStyle::DEFAULT, Stop.min)
+         .TieTextBox(XXO("St&op threshold (samples):"), mStop, 10);
    }
    S.EndMultiColumn();
 }
 
-bool EffectFindClipping::TransferDataToWindow()
+bool EffectFindClipping::TransferDataToWindow(const EffectSettings &)
 {
    ShuttleGui S(mUIParent, eIsSettingToDialog);
-   PopulateOrExchange(S);
+   // To do: eliminate this and just use validators for controls
+   DoPopulateOrExchange(S, *mpAccess);
 
    return true;
 }
 
-bool EffectFindClipping::TransferDataFromWindow()
+bool EffectFindClipping::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate())
    {
@@ -272,7 +258,8 @@ bool EffectFindClipping::TransferDataFromWindow()
    }
 
    ShuttleGui S(mUIParent, eIsGettingFromDialog);
-   PopulateOrExchange(S);
+   // To do: eliminate this and just use validators for controls
+   DoPopulateOrExchange(S, *mpAccess);
 
    return true;
 }
